#!/bin/sh
set +x
MODE=${1:-"scan"}

temp_log_file="/tmp/clamav_scan_$(date +%Y%m%d_%H%M%S).log"

total_files_estimated=0
total_files_scanned=0

create_temp_log() {
    local log_file="$1"
    cat "$log_file" > "$temp_log_file"
}

prepare_estimation() {
    echo "Preparing file count estimation... (Note: This is an estimate and may include files across filesystems or symlinks if not all conditions are respected)"
    local scan_dirs=$(find /scan -type d -print) # Lists all directories to scan

    # Use parallel to count files across directories, matching clamscan criteria and ignoring symlinks
    total_files_estimated=$(echo "$scan_dirs" | parallel -j $(nproc) \
        "find {} -type f -not -type l -size -100M | wc -l" | \
        awk '{s+=$1} END {print s}')

    echo "Estimated total files to scan: $total_files_estimated"
}

scan() {
    prepare_estimation

    echo "Running ClamAV scan on each subdirectory sequentially..."
    local scan_dirs=$(find /scan -type d -print) # Lists all directories to scan

    # Run clamscan on each subdirectory in sequence, ignoring symlinks
    for dir in $scan_dirs; do
        clamscan -r --max-scansize=100M --max-filesize=50M --max-recursion=0 --follow-dir-symlinks=no --cross-filesystems=no --log=/var/log/clamav/clamav_${dir##*/}.log "$dir"
        scanned_files=$(grep -E "^/.*: (OK|FOUND)" /var/log/clamav/clamav_${dir##*/}.log | wc -l)
        total_files_scanned=$((total_files_scanned + scanned_files))
        remaining_files=$((total_files_estimated - total_files_scanned))
        echo "Progress: $total_files_scanned files scanned, $remaining_files files remaining"
    done

    # Combine logs
    cat /var/log/clamav/clamav_*.log > /var/log/clamav/clamav.log
    rm /var/log/clamav/clamav_*.log
}

report() {
    echo "Generating report..."
    local log_file="/var/log/clamav/clamav.log"
    if [ ! -f "$log_file" ]; then
        echo "No log file found."
        return
    fi
    local total_files=$(grep "Scanned files:" "$log_file" | cut -d":" -f2 | xargs)
    local infected_files=$(grep "Infected files:" "$log_file" | cut -d":" -f2 | xargs)
    local errors=$(grep "Total errors:" "$log_file" | cut -d":" -f2 | xargs)
    echo "Scan Report:"
    echo "Total files scanned: $total_files"
    echo "Infected files found: $infected_files"
    echo "Errors during scan: $errors"

    if [ "$infected_files" -gt 0 ]; then
        echo "Reporting detections to GlitchTip..."
        create_temp_log "$log_file"
        go-glitch report --dsn "$GLITCHTIP_DSN" "$temp_log_file" || echo "Failed to report to GlitchTip"
    fi
}

quarantine() {
    echo "Quarantining infected files..."
    local log_file="/var/log/clamav/clamav.log"
    local quarantine_dir="/quarantine"
    mkdir -p "$quarantine_dir"
    grep "FOUND" "$log_file" | cut -d":" -f1 | while read -r infected_file; do
        if [ -f "$infected_file" ]; then
            mv "$infected_file" "$quarantine_dir/"
        fi
    done
}

case "$MODE" in
    scan)
        scan
        ;;
    report)
        report
        ;;
    quarantine)
        quarantine
        ;;
    *)
        echo "Unknown mode: $MODE"
        exit 1
        ;;
esac
